'use strict';

var react = require('react');
var jsxRuntime = require('react/jsx-runtime');

const wrap = (_ref, rest) => {
  let {
    Component,
    useHook,
    name
  } = _ref;
  const WithTogglePointPlugin = /*#__PURE__*/react.forwardRef((props, ref) => {
    useHook(rest);
    return /*#__PURE__*/jsxRuntime.jsx(Component, {
      ...props,
      ref: ref
    });
  });
  WithTogglePointPlugin.displayName = `With${name}(${Component.displayName || Component.name || "Component"})`;
  return WithTogglePointPlugin;
};
const withPlugins = _ref2 => {
  let {
    Component,
    plugins,
    ...rest
  } = _ref2;
  for (const {
    onCodeSelected: useHook,
    name
  } of plugins) {
    Component = wrap({
      Component,
      useHook,
      name
    }, rest);
  }
  return Component;
};

function _typeof(o) {
  "@babel/helpers - typeof";

  return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) {
    return typeof o;
  } : function (o) {
    return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o;
  }, _typeof(o);
}

function toPrimitive(t, r) {
  if ("object" != _typeof(t) || !t) return t;
  var e = t[Symbol.toPrimitive];
  if (void 0 !== e) {
    var i = e.call(t, r || "default");
    if ("object" != _typeof(i)) return i;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return ("string" === r ? String : Number)(t);
}

function toPropertyKey(t) {
  var i = toPrimitive(t, "string");
  return "symbol" == _typeof(i) ? i : i + "";
}

function _defineProperty(e, r, t) {
  return (r = toPropertyKey(r)) in e ? Object.defineProperty(e, r, {
    value: t,
    enumerable: !0,
    configurable: !0,
    writable: !0
  }) : e[r] = t, e;
}

const ForwardedRefContext = /*#__PURE__*/react.createContext();
const withErrorBoundary = _ref => {
  let {
    Variant,
    fallback,
    logError
  } = _ref;
  class TogglePointErrorBoundary extends react.Component {
    constructor(props) {
      super(props);
      this.state = {
        hasError: false
      };
    }
    static getDerivedStateFromError() {
      return {
        hasError: true
      };
    }
    componentDidCatch(error) {
      error.message = `Variant errored, rendering fallback: ${error.message}`;
      logError(error);
    }
    render() {
      const Component = this.state.hasError ? fallback : Variant;
      return /*#__PURE__*/jsxRuntime.jsx(Component, {
        ...this.props,
        ref: this.context
      });
    }
  }
  _defineProperty(TogglePointErrorBoundary, "contextType", ForwardedRefContext);
  const TogglePointErrorBoundaryWithRef = /*#__PURE__*/react.forwardRef((props, ref) => /*#__PURE__*/jsxRuntime.jsx(ForwardedRefContext.Provider, {
    value: ref,
    children: /*#__PURE__*/jsxRuntime.jsx(TogglePointErrorBoundary, {
      ...props
    })
  }));
  TogglePointErrorBoundaryWithRef.displayName = `withErrorBoundary(${Variant.displayName || Variant.name || "Component"})`;
  return TogglePointErrorBoundaryWithRef;
};

const getControlOrVariant = _ref => {
  let {
    matchedFeatures,
    matchedVariant,
    logError,
    control
  } = _ref;
  if (!matchedFeatures.length) {
    return control;
  }
  let Component = control;
  if (matchedVariant) {
    const {
      codeRequest,
      variables
    } = matchedVariant;
    const {
      default: VariantWithoutVariables
    } = codeRequest;
    const Variant = /*#__PURE__*/react.forwardRef((props, ref) => /*#__PURE__*/jsxRuntime.jsx(VariantWithoutVariables, {
      ...variables,
      ...props,
      ref
    }));
    Variant.displayName = `Variant(${VariantWithoutVariables.displayName || VariantWithoutVariables.name || "Component"})`;
    Component = withErrorBoundary({
      Variant,
      logError,
      fallback: control
    });
  }
  return Component;
};
const getComponent = params => {
  let Component = getControlOrVariant(params);
  const {
    plugins,
    ...rest
  } = params;
  if (plugins) {
    Component = withPlugins({
      Component,
      plugins,
      ...rest
    });
  }
  return Component;
};

const getMatchedVariant = _ref => {
  let {
    matchedFeatures,
    featuresMap,
    variantKey
  } = _ref;
  for (const [feature, {
    [variantKey]: variant,
    ...variables
  }] of matchedFeatures) {
    var _featuresMap$get;
    const codeRequest = (_featuresMap$get = featuresMap.get(feature)) === null || _featuresMap$get === void 0 ? void 0 : _featuresMap$get.get(variant);
    if (codeRequest) {
      return {
        codeRequest,
        variables
      };
    }
  }
  return null;
};

const getMatchedFeatures = _ref => {
  let {
    activeFeatures,
    featuresMap
  } = _ref;
  const matchedFeatures = Object.entries(activeFeatures || {}).filter(_ref2 => {
    let [key] = _ref2;
    return featuresMap.has(key);
  });
  return matchedFeatures;
};

const useCodeMatches = _ref => {
  let {
    activeFeatures,
    variantKey,
    featuresMap
  } = _ref;
  const seriazlizedActiveFeatures = JSON.stringify(activeFeatures);
  const matches = react.useMemo(() => {
    const matchedFeatures = getMatchedFeatures({
      activeFeatures,
      featuresMap
    });
    const matchedVariant = getMatchedVariant({
      matchedFeatures,
      featuresMap,
      variantKey
    });
    return {
      matchedFeatures,
      matchedVariant
    };
  }, [seriazlizedActiveFeatures]); // eslint-disable-line react-hooks/exhaustive-deps

  return matches;
};

/**
 * A plugin for the point cuts package
 * @typedef {object} module:web-toggle-point-react-pointcuts~plugin
 * @property {string} name plugin name, used as a prefix when creating {@link https://legacy.reactjs.org/docs/higher-order-components.html|React Higher-Order-Components} when toggling
 * @property {Function} onCodeSelected hook to be called when a code selection is made
 */

const getCodeSelectionPlugins = plugins => plugins.filter(_ref => {
  let {
    onCodeSelected
  } = _ref;
  return !!onCodeSelected;
});

/**
 * A factory function used to create a withTogglePoint React Higher-Order-Component.
 * @memberof module:web-toggle-point-react-pointcuts
 * @inner
 * @function
 * @param {object} params parameters
 * @param {function} params.getActiveFeatures a method to get active features. Called honouring the rules of hooks.
 * @param {external:HostApplication.logError} params.logError a method that logs errors
 * @param {string} [params.variantKey=bucket] A key use to identify a variant from the features data structure. Remaining members of the feature will be passed to the variant as props.
 * @param {Array<module:web-toggle-point-react-pointcuts~plugin>} [params.plugins] plugins to be used when toggling
 * Will be used when a toggled component throws an error that can be caught by an {@link https://reactjs.org/docs/error-boundaries.html|ErrorBoundary}.
 * When errors are caught, the control/base code will be used as the fallback component.
 * @returns {module:web-toggle-point-react-pointcuts.withTogglePoint} withTogglePoint React Higher-Order-Component.
 * @example
 * const withTogglePoint = withTogglePointFactory({
 *   getActiveFeatures,
 *   plugins: [plugin1, plugin2, plugin3],
 *   logError: (error) => window.NREUM?.noticeError(error)
 * });
 * export default withTogglePoint(MyReactComponent);
 */
const withTogglePointFactory = _ref => {
  let {
    getActiveFeatures,
    logError,
    variantKey = "bucket",
    plugins = []
  } = _ref;
  const codeSelectionPlugins = getCodeSelectionPlugins(plugins);

  /**
   * A React Higher-Order-Component that wraps a base / control component and swaps in a variant when deemed appropriate by a context
   * @function withTogglePoint
   * @memberof module:web-toggle-point-react-pointcuts
   * @param {ReactComponentModuleNamespaceObject} controlModule The control / base module
   * @param {external:React.Component} controlModule.default The control react component
   * @param {Map} featuresMap A map of features and their variants, with features as top-level keys and variants as nested keys with modules as the values.
   * @returns {external:React.Component} Wrapped react component
   */
  const withTogglePoint = (controlModule, featuresMap) => {
    const {
      default: control
    } = controlModule;
    const WithTogglePoint = /*#__PURE__*/react.forwardRef((props, ref) => {
      const activeFeatures = getActiveFeatures();
      const {
        matchedFeatures,
        matchedVariant
      } = useCodeMatches({
        featuresMap,
        variantKey,
        activeFeatures
      });
      const Component = react.useMemo(() => getComponent({
        matchedFeatures,
        matchedVariant,
        logError,
        control,
        plugins: codeSelectionPlugins
      }), [matchedFeatures, matchedVariant]);
      return /*#__PURE__*/jsxRuntime.jsx(Component, {
        ...props,
        ref: ref
      });
    });
    WithTogglePoint.displayName = `withTogglePoint(${control.displayName || control.name || "Component"})`;
    return WithTogglePoint;
  };
  return withTogglePoint;
};

const pluginsHookFactory = plugins => {
  const usePlugins = plugins.reduceRight((out, _ref) => {
    let {
      onCodeSelected
    } = _ref;
    return function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      onCodeSelected(...args);
      out === null || out === void 0 || out(...args);
    };
  }, null);
  return usePlugins;
};

/**
 * A factory function used to create a withToggledHook React hook, wrapping an existing hook/function.
 * @memberof module:web-toggle-point-react-pointcuts
 * @inner
 * @function
 * @param {object} params parameters
 * @param {function} params.getActiveFeatures a method to get active features, which is called honouring the rules of hooks.
 * @param {Array<module:web-toggle-point-react-pointcuts~plugin>} [params.plugins] plugins to be used when toggling
 * @returns {module:web-toggle-point-react-pointcuts.withToggledHook} withToggledHook hook function, use to wrap a function (either a hook itself, or a function that must be called wherever a hook can...).
 * @example
 * const getActiveFeatures = () => useContext(myContext);
 * const withToggledHook = withToggledHookFactory({
 *   getActiveFeatures,
 *   plugins: [plugin1, plugin2, plugin3]
 * });
 * export default withToggledHook(useMyHook);
 */
const withToggledHookFactory = _ref => {
  let {
    getActiveFeatures,
    plugins = []
  } = _ref;
  const codeSelectionPlugins = getCodeSelectionPlugins(plugins);
  const useCodeSelectionPlugins = pluginsHookFactory(codeSelectionPlugins);

  /**
   * A React hook that wraps a base / control function or hook and swaps in a variant when deemed appropriate by a context
   * @function withToggledHook
   * @memberof module:web-toggle-point-react-pointcuts
   * @param {ReactHookModuleNamespaceObject} controlModule The control / base module
   * @param {(external:React.Hook|function)} controlModule.default The control react hook or function.
   * @param {Map} featuresMap A map of features and their variants, with features as top-level keys and variants as nested keys with modules as the values.
   * @returns {external:React.Hook} Wrapped function / hook, as a hook (so must be applied in accordance with the {@link https://reactjs.org/docs/hooks-rules.html|rules of hooks})
   */
  const withToggledHook = (controlModule, featuresMap) => {
    const useTogglePoint = function () {
      var _matchedVariant$codeR;
      const activeFeatures = getActiveFeatures();
      const {
        matchedVariant
      } = useCodeMatches({
        featuresMap,
        activeFeatures
      });
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      useCodeSelectionPlugins === null || useCodeSelectionPlugins === void 0 || useCodeSelectionPlugins(...args);
      const {
        default: hook
      } = (_matchedVariant$codeR = matchedVariant === null || matchedVariant === void 0 ? void 0 : matchedVariant.codeRequest) !== null && _matchedVariant$codeR !== void 0 ? _matchedVariant$codeR : controlModule;
      return hook(...args);
    };
    return useTogglePoint;
  };
  return withToggledHook;
};

/**
 * Code expected in the host application
 * @external HostApplication
 */
/**
 * A delegate passed to log an error
 * @callback external:HostApplication.logError
 * @type {Function}
 * @param {string} message A message to log as an error
 */

/**
 * React UI library
 * @external React
 * @see {@link https://reactjs.org/|React}
 */
/**
 * @typedef {import("react").Component} Component
 * @memberof external:React
 * @see {@link https://reactjs.org/docs/react-component.html|React.Component}
 */
/**
 * @typedef {Function} Hook
 * @memberof external:React
 * @see {@link https://reactjs.org/docs/hooks-overview.html|React.Hook}
 */
/**
 * @typedef {import("react").Context} Context
 * @memberof external:React
 * @see {@link https://reactjs.org/docs/context.html|context object}
 */
/**
 * A {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/import#module_namespace_object|module namespace object} that exports a {@link external:React.Component|React Component} as its default export
 * @typedef {object} ReactComponentModuleNamespaceObject
 * @property {external:React.Component} default - The default export of the module, which should be a {@link external:React.Component|React Component}.
 */
/**
 * A {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/import#module_namespace_object|module namespace object} that exports a function as its default export, that can be a {@link external:React.Hook|React hook}
 * @typedef {object} ReactHookModuleNamespaceObject
 * @property {external:React.Hook} default - The default export of the module, typically a {@link external:React.Hook|React hook}.
 */

exports.withTogglePointFactory = withTogglePointFactory;
exports.withToggledHookFactory = withToggledHookFactory;
//# sourceMappingURL=main.es5.cjs.map
